import pandas as pd
import os
import time
import logging
import site
from MRItaxonomy.NCBI_fetch import initialize

TAXID2NAME_df = None
TAXID2NAME_df2 = None

def load_dataframe():
    logger = logging.getLogger(__name__)
    global TAXID2NAME_df
    global TAXID2NAME_df2
    if TAXID2NAME_df is None:

        site_packages = site.getsitepackages()[0]
        for root, dirs, files in os.walk(site_packages):
            if "MRItaxonomy" in dirs:
                nodedb_path = os.path.join(root, "MRItaxonomy")
                break
            else:
                print("Bad installation location")
                raise SystemExit
        nodedb_path=nodedb_path+"/dumps/"
        
        if os.path.exists(nodedb_path+"names.dmp"):
            logger.warning("Names database was last modified: {}".format(time.ctime(os.path.getctime(nodedb_path+"names.dmp"))))
        else:
            initialize()
            if os.path.exists(nodedb_path+"names.dmp"):
                logger.warning("Names database was last modified: {}".format(time.ctime(os.path.getctime(nodedb_path+"names.dmp"))))
            else:
                logger.critical("Database file not found.")
                raise SystemExit
        
        
        headers = ("tax_id", "name","type")
        TAXID2NAME_df = pd.read_table(nodedb_path+"names.dmp", sep="|", names=headers,index_col=0,usecols=(0,1,3))
        TAXID2NAME_df["name"] = TAXID2NAME_df["name"].str.strip()
        TAXID2NAME_df["type"] = TAXID2NAME_df["type"].str.strip()
        TAXID2NAME_df = TAXID2NAME_df[TAXID2NAME_df['type'].str.contains('scientific name')]
        mergedheaders = ("original", "merged")
        TAXID2NAME_df2 = pd.read_table(nodedb_path+"merged.dmp", sep="|", names=mergedheaders,index_col=0,usecols=(0,1))
    
    return TAXID2NAME_df, TAXID2NAME_df2


def get_merge(tid):
    TAXID2NAME_df, TAXID2NAME_df2 = load_dataframe()
    if tid not in TAXID2NAME_df.index:
        if tid in TAXID2NAME_df2.index:
            return TAXID2NAME_df2.loc[tid]['merged']
        else:
            return 0
    else:
	    return tid
	
def get_name(tax):
    TAXID2NAME_df, _ = load_dataframe()
    tax = get_merge(tax)
    return TAXID2NAME_df.loc[tax]['name']



import os
import wget
import site
import pandas as pd
import numpy as np
import marisa_trie
import subprocess

#functions called by commands at command line

def build_trie(directory):
    print('Building accession2taxid trie datafiles...')
    chunk_size = 50000
    trie_keys = []
    tax_ids = []
    for chunk in pd.read_csv(f'{directory}/dumps/nucl_gb.accession2taxid', sep='\t', usecols=[1, 2], chunksize=chunk_size, header=0):
        #print(chunk)
        #print(chunk.iloc[:,1].astype(int).tolist())
        #assert(False)
        if all(isinstance(item, int) for item in chunk.iloc[:,1].astype(int).tolist()):
            trie_keys.extend(chunk.iloc[:,0].astype(str).tolist())
            tax_ids.extend(chunk.iloc[:,1].astype(int).tolist())
            #print("Taxids completed: ",len(tax_ids))
        else:
            #print(chunk)
            #print(chunk.iloc[:,1].astype(int).tolist())
            #chunk[2] = pd.to_numeric(chunk[2], errors='coerce')
            #filtered_chunk = chunk.dropna(subset=[2])
            #trie_keys.extend(chunk[1].astype(str).tolist())
            #tax_ids.extend(chunk[2].tolist())
            #print("Taxids done post clean :",len(tax_ids))
            continue
    
    trie = marisa_trie.Trie(trie_keys)
    trie_indices = [trie[k] for k in trie_keys]
    ordered_tax_ids = [x for _, x in sorted(zip(trie_indices, tax_ids))]
    tax_ids_array = np.array(ordered_tax_ids, dtype=np.int32)
    np.save(f'{directory}/dumps/ordered_tax_ids.npy', tax_ids_array)
    trie.save(f'{directory}/dumps/accession_trie.marisa')

def initialize():
    site_packages = site.getsitepackages()[0]
    for root, dirs, files in os.walk(site_packages):
        if "MRItaxonomy" in dirs:
            directory = os.path.join(root, "MRItaxonomy")
            break
        else:
            print("Bad installation location")
            raise SystemExit

    if not os.path.exists(f'{directory}/dumps'):
        os.makedirs(f'{directory}/dumps') #make sure this is here

    print('Initializing...')
    if not os.path.exists(f'{directory}/dumps/new_taxdump.tar.gz.md5'):
        wget.download('https://ftp.ncbi.nlm.nih.gov/pub/taxonomy/new_taxdump/new_taxdump.tar.gz.md5', out=f'{directory}/dumps/new_taxdump.tar.gz.md5')
    if not os.path.exists(f'{directory}/dumps/new_taxdump.tar.gz'):
        wget.download('https://ftp.ncbi.nlm.nih.gov/pub/taxonomy/new_taxdump/new_taxdump.tar.gz', out=f'{directory}/dumps/new_taxdump.tar.gz')
    print(f'\nTaxonomy dump files downloaded to {directory}/dumps.')
    if not os.path.exists(f'{directory}/dumps/nucl_gb.accession2taxid.gz'):
        wget.download('https://ftp.ncbi.nlm.nih.gov/pub/taxonomy/accession2taxid/nucl_gb.accession2taxid.gz', out=f'{directory}/dumps/nucl_gb.accession2taxid.gz')
    if not os.path.exists(f'{directory}/dumps/nucl_gb.accession2taxid.gz.md5'):
        wget.download('https://ftp.ncbi.nlm.nih.gov/pub/taxonomy/accession2taxid/nucl_gb.accession2taxid.gz.md5', out=f'{directory}/dumps/nucl_gb.accession2taxid.gz.md5')
    if not os.path.exists(f'{directory}/dumps/prot.accession2taxid.gz'):
        wget.download('https://ftp.ncbi.nlm.nih.gov/pub/taxonomy/accession2taxid/prot.accession2taxid.gz', out=f'{directory}/dumps/prot.accession2taxid.gz')
    if not os.path.exists(f'{directory}/dumps/prot.accession2taxid.gz.md5'):
        wget.download('https://ftp.ncbi.nlm.nih.gov/pub/taxonomy/accession2taxid/prot.accession2taxid.gz.md5', out=f'{directory}/dumps/prot.accession2taxid.gz.md5')
    print(f'\nAccession2taxid dump files downloaded to {directory}/dumps.')
    #could replace these with the gzip and tarfile modules, but they return file and tarfile objects, so this os.subprocess is cleaner. Change this if portability becomes an issue
    #os.system(f'gunzip -c {directory}/dumps/nucl_gb.accession2taxid.gz > {directory}/dumps/nucl_gb.accession2taxid')
    print('Unzipping nucl_gb.accession2taxid.gz')
    subprocess.run(f'gunzip -c {directory}/dumps/nucl_gb.accession2taxid.gz > {directory}/dumps/nucl_gb.accession2taxid', shell=True, text=True, capture_output=True)
    #os.system(f'gunzip -c {directory}/dumps/prot.accession2taxid.gz > {directory}/dumps/prot.accession2taxid')
    print('Unzipping prot.accession2taxid.gz')
    subprocess.run(f'gunzip -c {directory}/dumps/prot.accession2taxid.gz > {directory}/dumps/prot.accession2taxid', shell=True, text=True, capture_output=True)
    #os.system(f'tar -C {directory}/dumps -xzf {directory}/dumps/new_taxdump.tar.gz')
    print('Untarring new_taxdump.tar.gz')
    subprocess.run(f'tar -C {directory}/dumps -xzf {directory}/dumps/new_taxdump.tar.gz', shell=True, text=True, capture_output=True)
    build_trie(directory)

    
def update():
    print('Updating all databases')
    site_packages = site.getsitepackages()[0]
    for root, dirs, files in os.walk(site_packages):
        if "MRItaxonomy" in dirs:
            directory = os.path.join(root, "MRItaxonomy")
            break
        else:
            print("Bad installation location")
            raise SystemExit
    if not os.path.exists(f'{directory}/dumps'):
        os.makedirs(f'{directory}/dumps') #make sure this is here

    #os.system(f'md5sum {directory}/dumps/nucl_gb.accession2taxid.gz | cut -d\  -f1 > {directory}/dumps/old.md5') #generate md5sum and push to file. Python doesn't have an elegant way to make md5s or compare them
    subprocess.run(f'md5sum {directory}/dumps/nucl_gb.accession2taxid.gz | cut -d\  -f1 > {directory}/dumps/old.md5', shell=True, text=True, capture_output=True)
    with open(f'{directory}/dumps/old.md5') as f:
        old5 = f.readlines()[0].strip('\n') #get the md5 of the existing file
    os.remove(f'{directory}/dumps/old.md5') #remove temp file
    
    if os.path.exists(f'{directory}/dumps/nucl_gb.accession2taxid.gz.md5'):
        os.remove(f'{directory}/dumps/nucl_gb.accession2taxid.gz.md5')
        
    wget.download('https://ftp.ncbi.nlm.nih.gov/pub/taxonomy/accession2taxid/nucl_gb.accession2taxid.gz.md5', out=f'{directory}/dumps/nucl_gb.accession2taxid.gz.md5') #download md5 for comparison to existing file
    #os.rename('nucl_gb.accession2taxid.gz.md5', '{0}/dumps/nucl_gb.accession2taxid.gz.md5') #move it
    with open(f'{directory}/dumps/nucl_gb.accession2taxid.gz.md5') as f: #open it and pull the md5 hash from the file
        new5 = f.readlines()[0].split(' ')[0] #because it also contains the filename
    if new5 == old5: #compare
        print('\nThe accession dump files are up to date.\n') #you're done. Congratulations on being up to date

    else: #you got work to do
        if os.path.exists(f'{directory}/dumps/nucl_gb.accession2taxid.gz'):
            os.remove(f'{directory}/dumps/nucl_gb.accession2taxid.gz')
        wget.download('https://ftp.ncbi.nlm.nih.gov/pub/taxonomy/accession2taxid/nucl_gb.accession2taxid.gz', out=f'{directory}/dumps/nucl_gb.accession2taxid.gz')
        #os.rename('nucl_gb.accession2taxid.gz', '{0}/dumps/nucl_gb.accession2taxid.gz')
        
        #os.system(f'gunzip {directory}/dumps/nucl_gb.accession2taxid.gz')
        subprocess.run(f'gunzip -c {directory}/dumps/nucl_gb.accession2taxid.gz > {directory}/dumps/nucl_gb.accession2taxid', shell=True, text=True, capture_output=True)
        print(f'\nUpdated accession2taxid dump files downloaded to {directory}/dumps.\n')
        build_trie(directory)


    #os.system(f'md5sum {directory}/dumps/new_taxdump.tar.gz | cut -d\  -f1 > {directory}/dumps/old.md5') #generate md5sum and push to file. Python doesn't have an elegant way to make md5s or compare them
    subprocess.run(f'md5sum {directory}/dumps/new_taxdump.tar.gz | cut -d\  -f1 > {directory}/dumps/old.md5',shell=True, text=True, capture_output=True)
    with open(f'{directory}/dumps/old.md5') as f:
        old5 = f.readlines()[0].strip('\n') #get the md5 of the existing file
    os.remove(f'{directory}/dumps/old.md5') #remove temp file
    
    if os.path.exists(f'{directory}/dumps/new_taxdump.tar.gz.md5'):
        os.remove(f'{directory}/dumps/new_taxdump.tar.gz.md5')
        
    wget.download('https://ftp.ncbi.nlm.nih.gov/pub/taxonomy/new_taxdump/new_taxdump.tar.gz.md5', out=f'{directory}/dumps/new_taxdump.tar.gz.md5')
    #os.rename('new_taxdump.tar.gz.md5', '{0}/dumps/new_taxdump.tar.gz.md5')
    with open(f'{directory}/dumps/new_taxdump.tar.gz.md5') as f: #open it and pull the md5 hash from the file
        new5 = f.readlines()[0].split(' ')[0] #because it also contains the filename
    if new5 == old5: #compare
        print('\nThe taxonomy dump files are up to date.\n') #you're done. Congratulations on being up to date
    else:
        if os.path.exists(f'{directory}/dumps/new_taxdump.tar.gz'):
            os.remove(f'{directory}/dumps/new_taxdump.tar.gz')
        wget.download('https://ftp.ncbi.nlm.nih.gov/pub/taxonomy/new_taxdump/new_taxdump.tar.gz', out=f'{directory}/dumps/new_taxdump.tar.gz')
        #os.rename('new_taxdump.tar.gz', '{0}/dumps/new_taxdump.tar.gz')
        #os.system(f'tar -C {directory}/dumps -xzf {directory}/dumps/new_taxdump.tar.gz')
        subprocess.run(f'tar -C {directory}/dumps -xzf {directory}/dumps/new_taxdump.tar.gz', shell=True, text=True, capture_output=True)
        print(f'\nUpdated taxonomy dump files downloaded to {directory}/dumps.')


    subprocess.run(f'md5sum {directory}/dumps/prot.accession2taxid.gz | cut -d\  -f1 > {directory}/dumps/old.md5',shell=True, text=True, capture_output=True)
    with open(f'{directory}/dumps/old.md5') as f:
        old5 = f.readlines()[0].strip('\n') #get the md5 of the existing file
    os.remove(f'{directory}/dumps/old.md5') #remove temp file

    if os.path.exists(f'{directory}/dumps/prot.accession2taxid.gz.md5'):
        os.remove(f'{directory}/dumps/prot.accession2taxid.gz.md5')

    wget.download('https://ftp.ncbi.nlm.nih.gov/pub/taxonomy/accession2taxid/prot.accession2taxid.gz.md5', out=f'{directory}/dumps/prot.accession2taxid.gz.md5')
    with open(f'{directory}/dumps/prot.accession2taxid.gz.md5') as f: #open it and pull the md5 hash from the file
        new5 = f.readlines()[0].split(' ')[0] #because it also contains the filename
    if new5 == old5: #compare
        print('\nThe prot accession2taxid dump files are up to date.\n') #you're done. Congratulations on being up to date
    else:
        if os.path.exists(f'{directory}/dumps/prot.accession2taxid.gz'):
            os.remove(f'{directory}/dumps/prot.accession2taxid.gz')
        wget.download('https://ftp.ncbi.nlm.nih.gov/pub/taxonomy/accession2taxid/prot.accession2taxid.gz', out=f'{directory}/dumps/prot.accession2taxid.gz')
        subprocess.run(f'gunzip -c {directory}/dumps/prot.accession2taxid.gz > {directory}/dumps/prot.accession2taxid', shell=True, text=True, capture_output=True)
        print('\nUpdated prot accession2taxid dump files')
        










































import pandas as pd
import os
import time
import site
import logging

df = None
df2 = None

def load_dbs():
    logger = logging.getLogger(__name__)
    global df
    global df2
    if df is None or df2 is None:
        site_packages = site.getsitepackages()[0]
        for root, dirs, files in os.walk(site_packages):
            if "MRItaxonomy" in dirs:
                nodedb_path = os.path.join(root, "MRItaxonomy")
                break
            else:
                print("Bad installation location")
                raise SystemExit
        
        nodedb_path=nodedb_path+"/dumps/"
        if os.path.exists(nodedb_path+"nodes.dmp") and os.path.exists(nodedb_path+"merged.dmp"):
            logger.warning("Nodes database was last modified: {}".format(time.ctime(os.path.getctime(nodedb_path+"nodes.dmp"))))
            logger.warning("Merged database was last modified: {}".format(time.ctime(os.path.getctime(nodedb_path+"merged.dmp"))))
        else:
            logger.critical("One or more database files are missing. Run update.py to download.")
            raise SystemExit
        
        headers = ("tax_id", "parent", "rank")
        df = pd.read_table(nodedb_path+"nodes.dmp", sep="|", names=headers,index_col=0,usecols=(0,1,2))
        df["rank"] = df["rank"].str.strip()
        mergedheaders = ("original", "merged")
        df2 = pd.read_table(nodedb_path+"merged.dmp", sep="|", names=mergedheaders,index_col=0,usecols=(0,1))
    return df, df2


def getparent(tid):
    df, df2 = load_dbs()
    if tid in df.index:
        return df.loc[tid].parent
    elif tid in df2.index:
        return df.loc[df2.loc[tid]["merged"]].parent
    else:
        return 0
    

def getrank(tid):
    df, df2 = load_dbs()
    if tid in df.index:
        return df.loc[tid]["rank"]
    elif tid in df2.index:
        return df.loc[df2.loc[tid]["merged"]]["rank"]
    else:
        return "na"

def getnodeatrank(tid, selectedrank):
    currentrank = getrank(tid)
    if not getrank(tid) == 'na':

        while(currentrank != selectedrank and tid != 1):
            nextnode = getparent(tid)
            currentrank = getrank(nextnode)
            tid = nextnode
        else:
            if currentrank == selectedrank:
                return tid
            else:
                return 0

    else:
        return 0

def get_merge(tid):
    df, df2 = load_dbs()
    if tid not in df.index:
        if tid in df2.index:
            return df2.loc[tid]['merged']
        else:
            return 0
    else:
        return tid




from . import NCBI_fetch
from . import accession2taxid
from . import nnID
from . import nucDL
from . import protacc2taxid
from . import slidingwindow
from . import taxid
from . import taxid2name


# To initialize and or update databases
# from MRItaxonomy import NCBI_fetch 
# NCBI_fetch.initialize()
# from MRItaxonomy import NCBI_fetch 
# NCBI_fetch.update()


import pandas as pd
import os
import time
import logging
import site
from MRItaxonomy.NCBI_fetch import initialize

PROTACC2TAXID_df = None

def load_dataframe():
    logger = logging.getLogger(__name__)
    global PROTACC2TAXID_df
    if PROTACC2TAXID_df is None:
        site_packages = site.getsitepackages()[0]
        for root, dirs, files in os.walk(site_packages):
            if "MRItaxonomy" in dirs:
                db_path = os.path.join(root, "MRItaxonomy")
                break
            else:
                print("Bad installation location")
                raise SystemExit
        db_path=db_path+"/dumps/"
        
        if os.path.exists(db_path+'prot.accession2taxid'):
            logger.warning("Accession2taxid database was last modified: {}".format(time.ctime(os.path.getctime(db_path+'prot.accession2taxid'))))
        else:
            initialize()
            if os.path.exists(db_path+'prot.accession2taxid'):
                logger.warning("Accession2taxid database was last modified: {}".format(time.ctime(os.path.getctime(db_path+'prot.accession2taxid'))))
            else:
                logger.critical("Database file not found.")
                raise SystemExit
        
        PROTACC2TAXID_df = pd.read_table(db_path+'prot.accession2taxid',index_col=0,usecols=(1,2))
    return PROTACC2TAXID_df


def get_taxid(a_num):
    PROTACC2TAXID_df = load_dataframe()
    try:
        return PROTACC2TAXID_df.loc[a_num].taxid
    except: return -1




"""
This is a module designed to download nucleotides from NCBI's ftp site
It takes taxon as an argument and has two optional arguments:
path - which defaults to the current working directory.
db - which defaults to 'genbank'. Alternate option is 'refseq'

It is now parallelized.
"""

from Bio import Entrez
import os, shutil, pandas as pd, sys, time
import glob
from MRItaxonomy import taxid
from multiprocessing.dummy import Pool
from urllib.error import HTTPError
Entrez.email = 'biofx@mriglobal.org'

def dl(tax,threads,path=os.getcwd(),db='genbank'):
    tax_list = []
#strain level taxIDs are in the assembly_summary_refseq/genbank file and could be parsed as well.
#to do this, check if rank is higher than species and if so, collect all species for the tax_list
#if the rank is species or strain, continue.
#if the rank is strain, the summary file needs to be parsed on the taxid column, not the species_taxid column.
    if taxid.getrank(tax) != 'species': #to be updated
        print('Getting species for {0}\n'.format(tax))
        i=0
        while i<10:
            i+=1
            try:
                tax_list = Entrez.read(Entrez.esearch(db='taxonomy',term='txid{0}[Organism] AND species[Rank]'.format(tax), retmax='1000000'))['IdList'] #gets all taxIDs below the parent rank appropriate to >
            except HTTPError as e:
                time.sleep(1)
                print(e.read())
                print('Download problem at taxid.getrank')
                continue
            break
        print(tax_list)
    else:
        tax_list.append(tax) #keep everything consistent

    if not os.path.exists(path):
        os.makedirs(path)
    os.chdir(path)

    if os.path.exists('{0}/assembly_summary_{1}.txt'.format(path,db)):
#Assuming we only want complete genomes and not assemblies/scaffolds
        os.system('grep "# assembly_accession\|Complete Genome" {0}/assembly_summary_{1}.txt | cut -f6,7,8,9,20 > {0}/assembly.summary.shrunk.txt'.format(path,db))
    else:
        ftp_url = 'ftp://ftp.ncbi.nlm.nih.gov/genomes/{0}/assembly_summary_{0}.txt'.format(db) #Full ftp path
        i=0 #in case of internet shenanigans
        while i<10:
            i+=1
            try:
                print('Downloading {0} ... attempt {1}'.format(ftp_url, i))
                os.system('wget -c {0}'.format(ftp_url)) #download!
                os.system('grep "# assembly_accession\|Complete Genome" {0}/assembly_summary_{1}.txt | cut -f6,7,8,9,20 > {0}/assembly.summary.shrunk.txt'.format(path,db))
                time.sleep(2)
            except:
                time.sleep(i*i)
                continue
            break


    filenames = [] #set list of files to download. Pushed before the for loop to help parallelize taxa

    for tax in tax_list:
        try:
            print('{0} is the path of the taxon {1}'.format(path,tax))
            tax = int(tax)
            sys.stderr.write('{0} is the downloading taxID\n'.format(tax))
            assaddr = pd.read_csv('{0}/assembly.summary.shrunk.txt'.format(path),sep='\t') #modified from previous to handle the smaller file for improved search speeds.
            ftpaddr = list(assaddr.loc[assaddr['species_taxid'] == tax, ['ftp_path']]['ftp_path'])

            for addr in ftpaddr:
                filenames.append('{0}/{1}_genomic.fna.gz'.format(addr,addr.split('/')[9])) #add to list of files to download both the URL and the filename (10th field /-delimited of the URL)
        except HTTPError as e:
            time.sleep(5)
            print("HTTPError at {0}".format(tax))
            continue
        except KeyError:
#try and handle any updates to refseq that didn't update the taxonomy file
            print('TaxID {0} not found in taxonomy dump files'.format(tax))
            break
#then download with wget or anything else
#parallelize here
    pool = Pool(int(threads))
    pool.map(download,filenames)

def download(file):
    if not os.path.isfile(file):
        os.system('wget -c {0}'.format(file)) #downloads to current directory
        time.sleep(1)
        print(file.split('/')[-1])



from setuptools import setup, find_packages

setup(
    name="MRItaxonomy",
    version="1.1.0",
    packages=find_packages(),
    install_requires=['pandas>=0.19.0',
                      'biopython>=1.7',
                      'wget>=3.2',
                      'marisa_trie>=1.1.0'],
    author="MRIGlobal Bioinformatics Team",
    author_email="biofx@mriglobal.org",
    keywords="mriglobal taxonomy ncbi",
    description="MRIGlobal's taxonomy related operators",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    license="MIT", # or the license you chose
    url="https://github.com/mriglobal/MRItaxonomy",
    classifiers=[
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python",
    ],
)



"""
This is a module designed to identify near neighbor taxa to an input taxon. It uses classical taxonomic ranks
instead of sub-phylum or other such ranks. These are shown here:

    "strain":"species",
        "species":"genus",
        "genus":"family",
        "family":"order",
        "order":"class",
        "class":"phylum",
        "phylum":"superkingdom"
This will not return the input taxon
"""

from MRItaxonomy import taxid
from Bio import Entrez
Entrez.email = 'biofx@mriglobal.org'


def get_id(taxon):

#set up taxonomic ranks dictionary to get the correct parent ID.
    ranks = {
        "no rank":"species",
        "strain":"species",
        "species":"genus",
        "genus":"family",
        "family":"order",
        "order":"class",
        "class":"phylum",
        "phylum":"superkingdom"
    }

    rank = taxid.getrank(int(taxon)) #get current rank to use in taxID query

    parent_rank = ranks[rank] #get next classical taxonomic rank up the tree
    parent_id = int(taxid.getnodeatrank(int(taxon),parent_rank)) #get parent id of whatever your input is

    tax_list = Entrez.read(Entrez.esearch(db='taxonomy',term='txid{0}[Organism] AND {1}[Rank]'.format(parent_id,rank), retmax='100000000'))['IdList'] #gets all taxIDs below the parent rank appropriate to the input rank
    if taxon in tax_list:
        tax_list.remove('{0}'.format(taxon)) #get rid of target taxon

    return(tax_list)




"""
This is a module designed to shred a genome into 'reads' of consistent length
along the entirety of the genome in a sliding window fashion.
Takes three arguments:
path is required. This refers to the directory of fastas
window_size defaults to 150. This refers to the length of the "read"
extension defaults to .fna. This refers to the input fasta file extension
"""

from Bio import SeqIO
import os

def window(fseq, window_size):
    for i in range(len(fseq) - window_size + 1):
        yield fseq[i:i+window_size]


def reads_generation(path,window_size=150,extension='.fna'):
#Identify the sequences from SequencePath argument or NN argument and generate in reads for each one at 100x coverage. Output to location.
#Note, NNs are not yet part of this script.
#Loop through sequences in directory, run dwgsim


    print("Generating reads by sliding window across genome")
    odirectory = (path + '/reads')
    if not os.path.exists(odirectory):
        os.makedirs(odirectory)

    print(odirectory)

    for file in os.listdir(path):
        if file.endswith(extension):
            print(file)
            input_file = (path + '/' + file)
            output_prefix = (os.path.basename(file).split(extension)[0])
            print(output_prefix)
            outputfile = ('{0}/{1}.sliding.fasta'.format(odirectory,output_prefix))

            with open(input_file,'r') as f:
                with open(outputfile,'w') as o:
                    for record in SeqIO.parse(f,'fasta'):
                        counter=0
                        for item in window(record.seq,window_size):
                            counter+=1
                            item=str(item)
                            id='{0}.{1}'.format(record.id,counter)
                            o.write(">{0}\n{1}\n".format(id,item))


